@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Nicira, Inc.
+ * Copyright (c) 2015, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,15 +25,15 @@
 #include "command-line.h"
 #include "compiler.h"
 #include "dirs.h"
-#include "dynamic-string.h"
+#include "openvswitch/dynamic-string.h"
 #include "fatal-signal.h"
 #include "hash.h"
-#include "json.h"
+#include "openvswitch/json.h"
 #include "openvswitch/vlog.h"
 #include "ovsdb-data.h"
 #include "ovsdb-idl.h"
 #include "ovsdb-idl-provider.h"
-#include "shash.h"
+#include "openvswitch/shash.h"
 #include "sset.h"
 #include "string.h"
 #include "table.h"
@@ -62,7 +62,7 @@ static void (*ctl_exit_func)(int status) = NULL;
 OVS_NO_RETURN static void ctl_exit(int status);
 
 /* Represents all tables in the schema.  User must define 'tables'
- * in implementation and supply via clt_init().  The definition must end
+ * in implementation and supply via ctl_init().  The definition must end
  * with an all-NULL entry. */
 static const struct ctl_table_class *tables;
 
@@ -254,18 +254,30 @@ get_row_by_id(struct ctl_context *ctx, const struct ctl_table_class *table,
             return NULL;
         }
     } else {
-        const struct ovsdb_idl_row *row;
-
         referrer = NULL;
-        for (row = ovsdb_idl_first_row(ctx->idl, id->table);
-             row != NULL;
-             row = ovsdb_idl_next_row(row))
-            {
-                const struct ovsdb_datum *name;
 
-                name = ovsdb_idl_get(row, id->name_column,
-                                     OVSDB_TYPE_STRING, OVSDB_TYPE_VOID);
-                if (name->n == 1 && !strcmp(name->keys[0].string, record_id)) {
+        ovs_assert(id->name_column->type.value.type == OVSDB_TYPE_VOID);
+
+        enum ovsdb_atomic_type key = id->name_column->type.key.type;
+        if (key == OVSDB_TYPE_INTEGER) {
+            if (!record_id[0] || record_id[strspn(record_id, "0123456789")]) {
+                return NULL;
+            }
+        } else {
+            ovs_assert(key == OVSDB_TYPE_STRING);
+        }
+
+        for (const struct ovsdb_idl_row *row = ovsdb_idl_first_row(ctx->idl,
+                                                                   id->table);
+             row != NULL;
+             row = ovsdb_idl_next_row(row)) {
+            const struct ovsdb_datum *name = ovsdb_idl_get(
+                row, id->name_column, key, OVSDB_TYPE_VOID);
+            if (name->n == 1) {
+                const union ovsdb_atom *atom = &name->keys[0];
+                if (key == OVSDB_TYPE_STRING
+                    ? !strcmp(atom->string, record_id)
+                    : atom->integer == strtoll(record_id, NULL, 10)) {
                     if (referrer) {
                         ctl_fatal("multiple rows in %s match \"%s\"",
                                   table->class->name, record_id);
@@ -273,6 +285,7 @@ get_row_by_id(struct ctl_context *ctx, const struct ctl_table_class *table,
                     referrer = row;
                 }
             }
+        }
     }
     if (!referrer) {
         return NULL;
@@ -1994,7 +2007,7 @@ ctl_fatal(const char *format, ...)
     message = xvasprintf(format, args);
     va_end(args);
 
-    vlog_set_levels(&VLM_db_ctl_base, VLF_CONSOLE, VLL_OFF);
+    vlog_set_levels(&this_module, VLF_CONSOLE, VLL_OFF);
     VLOG_ERR("%s", message);
     ovs_error(0, "%s", message);
     ctl_exit(EXIT_FAILURE);
