@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, 2013, 2014, 2015 Nicira, Inc.
+ * Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,15 +19,16 @@
 #include "learn.h"
 
 #include "byte-order.h"
-#include "dynamic-string.h"
-#include "match.h"
-#include "meta-flow.h"
+#include "colors.h"
 #include "nx-match.h"
-#include "ofp-actions.h"
-#include "ofp-errors.h"
-#include "ofp-util.h"
-#include "ofpbuf.h"
 #include "openflow/openflow.h"
+#include "openvswitch/dynamic-string.h"
+#include "openvswitch/match.h"
+#include "openvswitch/meta-flow.h"
+#include "openvswitch/ofp-actions.h"
+#include "openvswitch/ofp-errors.h"
+#include "openvswitch/ofp-util.h"
+#include "openvswitch/ofpbuf.h"
 #include "unaligned.h"
 
 
@@ -40,7 +41,7 @@ learn_check(const struct ofpact_learn *learn, const struct flow *flow)
     struct match match;
 
     match_init_catchall(&match);
-    for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
+    OFPACT_LEARN_SPEC_FOR_EACH (spec, learn) {
         enum ofperr error;
 
         /* Check the source. */
@@ -58,8 +59,10 @@ learn_check(const struct ofpact_learn *learn, const struct flow *flow)
             if (error) {
                 return error;
             }
-
-            mf_write_subfield(&spec->dst, &spec->src_imm, &match);
+            if (spec->src_type & NX_LEARN_SRC_IMMEDIATE) {
+                mf_write_subfield_value(&spec->dst,
+                                        ofpact_learn_spec_imm(spec), &match);
+            }
             break;
 
         case NX_LEARN_DST_LOAD:
@@ -110,7 +113,6 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
     }
     fm->ofpacts = NULL;
     fm->ofpacts_len = 0;
-    fm->delete_reason = OFPRR_DELETE;
 
     if (learn->fin_idle_timeout || learn->fin_hard_timeout) {
         struct ofpact_fin_timeout *oft;
@@ -120,14 +122,15 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
         oft->fin_hard_timeout = learn->fin_hard_timeout;
     }
 
-    for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
+    OFPACT_LEARN_SPEC_FOR_EACH (spec, learn) {
         struct ofpact_set_field *sf;
         union mf_subvalue value;
 
         if (spec->src_type == NX_LEARN_SRC_FIELD) {
             mf_read_subfield(&spec->src, flow, &value);
         } else {
-            value = spec->src_imm;
+            mf_subvalue_from_value(&spec->dst, &value,
+                                   ofpact_learn_spec_imm(spec));
         }
 
         switch (spec->dst_type) {
@@ -136,13 +139,12 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
             break;
 
         case NX_LEARN_DST_LOAD:
-            sf = ofpact_put_reg_load(ofpacts);
-            sf->field = spec->dst.field;
+            sf = ofpact_put_reg_load(ofpacts, spec->dst.field, NULL, NULL);
             bitwise_copy(&value, sizeof value, 0,
-                         &sf->value, spec->dst.field->n_bytes, spec->dst.ofs,
+                         sf->value, spec->dst.field->n_bytes, spec->dst.ofs,
                          spec->n_bits);
-            bitwise_one(&sf->mask, spec->dst.field->n_bytes, spec->dst.ofs,
-                        spec->n_bits);
+            bitwise_one(ofpact_set_field_mask(sf), spec->dst.field->n_bytes,
+                        spec->dst.ofs, spec->n_bits);
             break;
 
         case NX_LEARN_DST_OUTPUT:
@@ -161,7 +163,6 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
             break;
         }
     }
-    ofpact_pad(ofpacts);
 
     fm->ofpacts = ofpacts->data;
     fm->ofpacts_len = ofpacts->size;
@@ -176,7 +177,7 @@ learn_mask(const struct ofpact_learn *learn, struct flow_wildcards *wc)
     union mf_subvalue value;
 
     memset(&value, 0xff, sizeof value);
-    for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
+    OFPACT_LEARN_SPEC_FOR_EACH (spec, learn) {
         if (spec->src_type == NX_LEARN_SRC_FIELD) {
             mf_write_subfield_flow(&spec->src, &value, &wc->masks);
         }
@@ -186,7 +187,8 @@ learn_mask(const struct ofpact_learn *learn, struct flow_wildcards *wc)
 /* Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string. */
 static char * OVS_WARN_UNUSED_RESULT
-learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
+learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec,
+                           struct ofpbuf *ofpacts)
 {
     const char *full_s = s;
     struct mf_subfield dst;
@@ -221,9 +223,14 @@ learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
 
     spec->n_bits = dst.n_bits;
     spec->src_type = NX_LEARN_SRC_IMMEDIATE;
-    spec->src_imm = imm;
     spec->dst_type = NX_LEARN_DST_LOAD;
     spec->dst = dst;
+
+    /* Push value last, as this may reallocate 'spec'! */
+    unsigned int n_bytes = DIV_ROUND_UP(dst.n_bits, 8);
+    uint8_t *src_imm = ofpbuf_put_zeros(ofpacts, OFPACT_ALIGN(n_bytes));
+    memcpy(src_imm, &imm.u8[sizeof imm.u8 - n_bytes], n_bytes);
+
     return NULL;
 }
 
@@ -231,7 +238,8 @@ learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
  * error.  The caller is responsible for freeing the returned string. */
 static char * OVS_WARN_UNUSED_RESULT
 learn_parse_spec(const char *orig, char *name, char *value,
-                 struct ofpact_learn_spec *spec)
+                 struct ofpact_learn_spec *spec,
+                 struct ofpbuf *ofpacts, struct match *match)
 {
     if (mf_from_name(name)) {
         const struct mf_field *dst = mf_from_name(name);
@@ -245,13 +253,19 @@ learn_parse_spec(const char *orig, char *name, char *value,
 
         spec->n_bits = dst->n_bits;
         spec->src_type = NX_LEARN_SRC_IMMEDIATE;
-        memset(&spec->src_imm, 0, sizeof spec->src_imm);
-        memcpy(&spec->src_imm.u8[sizeof spec->src_imm - dst->n_bytes],
-               &imm, dst->n_bytes);
         spec->dst_type = NX_LEARN_DST_MATCH;
         spec->dst.field = dst;
         spec->dst.ofs = 0;
         spec->dst.n_bits = dst->n_bits;
+
+        /* Update 'match' to allow for satisfying destination
+         * prerequisites. */
+        mf_set_value(dst, &imm, match, NULL);
+
+        /* Push value last, as this may reallocate 'spec'! */
+        uint8_t *src_imm = ofpbuf_put_zeros(ofpacts,
+                                            OFPACT_ALIGN(dst->n_bytes));
+        memcpy(src_imm, &imm, dst->n_bytes);
     } else if (strchr(name, '[')) {
         /* Parse destination and check prerequisites. */
         char *error;
@@ -285,7 +299,7 @@ learn_parse_spec(const char *orig, char *name, char *value,
         spec->dst_type = NX_LEARN_DST_MATCH;
     } else if (!strcmp(name, "load")) {
         if (value[strcspn(value, "[-")] == '-') {
-            char *error = learn_parse_load_immediate(value, spec);
+            char *error = learn_parse_load_immediate(value, spec, ofpacts);
             if (error) {
                 return error;
             }
@@ -364,23 +378,14 @@ learn_parse__(char *orig, char *arg, struct ofpbuf *ofpacts)
             char *error;
 
             spec = ofpbuf_put_zeros(ofpacts, sizeof *spec);
-            learn = ofpacts->header;
-            learn->n_specs++;
-
-            error = learn_parse_spec(orig, name, value, spec);
+            error = learn_parse_spec(orig, name, value, spec, ofpacts, &match);
             if (error) {
                 return error;
             }
-
-            /* Update 'match' to allow for satisfying destination
-             * prerequisites. */
-            if (spec->src_type == NX_LEARN_SRC_IMMEDIATE
-                && spec->dst_type == NX_LEARN_DST_MATCH) {
-                mf_write_subfield(&spec->dst, &spec->src_imm, &match);
-            }
+            learn = ofpacts->header;
         }
     }
-    ofpact_update_len(ofpacts, &learn->ofpact);
+    ofpact_finish_LEARN(ofpacts, &learn);
 
     return NULL;
 }
@@ -416,82 +421,94 @@ learn_format(const struct ofpact_learn *learn, struct ds *s)
 
     match_init_catchall(&match);
 
-    ds_put_format(s, "learn(table=%"PRIu8, learn->table_id);
+    ds_put_format(s, "%slearn(%s%stable=%s%"PRIu8,
+                  colors.learn, colors.end, colors.special, colors.end,
+                  learn->table_id);
     if (learn->idle_timeout != OFP_FLOW_PERMANENT) {
-        ds_put_format(s, ",idle_timeout=%"PRIu16, learn->idle_timeout);
+        ds_put_format(s, ",%sidle_timeout=%s%"PRIu16,
+                      colors.param, colors.end, learn->idle_timeout);
     }
     if (learn->hard_timeout != OFP_FLOW_PERMANENT) {
-        ds_put_format(s, ",hard_timeout=%"PRIu16, learn->hard_timeout);
+        ds_put_format(s, ",%shard_timeout=%s%"PRIu16,
+                      colors.param, colors.end, learn->hard_timeout);
     }
     if (learn->fin_idle_timeout) {
-        ds_put_format(s, ",fin_idle_timeout=%"PRIu16, learn->fin_idle_timeout);
+        ds_put_format(s, ",%sfin_idle_timeout=%s%"PRIu16,
+                      colors.param, colors.end, learn->fin_idle_timeout);
     }
     if (learn->fin_hard_timeout) {
-        ds_put_format(s, ",fin_hard_timeout=%"PRIu16, learn->fin_hard_timeout);
+        ds_put_format(s, "%s,fin_hard_timeout=%s%"PRIu16,
+                      colors.param, colors.end, learn->fin_hard_timeout);
     }
     if (learn->priority != OFP_DEFAULT_PRIORITY) {
-        ds_put_format(s, ",priority=%"PRIu16, learn->priority);
+        ds_put_format(s, "%s,priority=%s%"PRIu16,
+                      colors.special, colors.end, learn->priority);
     }
     if (learn->flags & NX_LEARN_F_SEND_FLOW_REM) {
-        ds_put_cstr(s, ",send_flow_rem");
+        ds_put_format(s, ",%ssend_flow_rem%s", colors.value, colors.end);
     }
     if (learn->flags & NX_LEARN_F_DELETE_LEARNED) {
-        ds_put_cstr(s, ",delete_learned");
+        ds_put_format(s, ",%sdelete_learned%s", colors.value, colors.end);
     }
     if (learn->cookie != 0) {
-        ds_put_format(s, ",cookie=%#"PRIx64, ntohll(learn->cookie));
+        ds_put_format(s, ",%scookie=%s%#"PRIx64,
+                      colors.param, colors.end, ntohll(learn->cookie));
     }
 
-    for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
+    OFPACT_LEARN_SPEC_FOR_EACH (spec, learn) {
+        unsigned int n_bytes = DIV_ROUND_UP(spec->n_bits, 8);
         ds_put_char(s, ',');
 
         switch (spec->src_type | spec->dst_type) {
-        case NX_LEARN_SRC_IMMEDIATE | NX_LEARN_DST_MATCH:
+        case NX_LEARN_SRC_IMMEDIATE | NX_LEARN_DST_MATCH: {
             if (spec->dst.ofs == 0
                 && spec->dst.n_bits == spec->dst.field->n_bits) {
                 union mf_value value;
 
                 memset(&value, 0, sizeof value);
-                bitwise_copy(&spec->src_imm, sizeof spec->src_imm, 0,
-                             &value, spec->dst.field->n_bytes, 0,
-                             spec->dst.field->n_bits);
-                ds_put_format(s, "%s=", spec->dst.field->name);
+                memcpy(&value.b[spec->dst.field->n_bytes - n_bytes],
+                       ofpact_learn_spec_imm(spec), n_bytes);
+                ds_put_format(s, "%s%s=%s", colors.param,
+                              spec->dst.field->name, colors.end);
                 mf_format(spec->dst.field, &value, NULL, s);
             } else {
+                ds_put_format(s, "%s", colors.param);
                 mf_format_subfield(&spec->dst, s);
-                ds_put_char(s, '=');
-                mf_format_subvalue(&spec->src_imm, s);
+                ds_put_format(s, "=%s", colors.end);
+                ds_put_hex(s, ofpact_learn_spec_imm(spec), n_bytes);
             }
             break;
-
+        }
         case NX_LEARN_SRC_FIELD | NX_LEARN_DST_MATCH:
+            ds_put_format(s, "%s", colors.param);
             mf_format_subfield(&spec->dst, s);
+            ds_put_format(s, "%s", colors.end);
             if (spec->src.field != spec->dst.field ||
                 spec->src.ofs != spec->dst.ofs) {
-                ds_put_char(s, '=');
+                ds_put_format(s, "%s=%s", colors.param, colors.end);
                 mf_format_subfield(&spec->src, s);
             }
             break;
 
         case NX_LEARN_SRC_IMMEDIATE | NX_LEARN_DST_LOAD:
-            ds_put_format(s, "load:");
-            mf_format_subvalue(&spec->src_imm, s);
-            ds_put_cstr(s, "->");
+            ds_put_format(s, "%sload:%s", colors.special, colors.end);
+            ds_put_hex(s, ofpact_learn_spec_imm(spec), n_bytes);
+            ds_put_format(s, "%s->%s", colors.special, colors.end);
             mf_format_subfield(&spec->dst, s);
             break;
 
         case NX_LEARN_SRC_FIELD | NX_LEARN_DST_LOAD:
-            ds_put_cstr(s, "load:");
+            ds_put_format(s, "%sload:%s", colors.special, colors.end);
             mf_format_subfield(&spec->src, s);
-            ds_put_cstr(s, "->");
+            ds_put_format(s, "%s->%s", colors.special, colors.end);
             mf_format_subfield(&spec->dst, s);
             break;
 
         case NX_LEARN_SRC_FIELD | NX_LEARN_DST_OUTPUT:
-            ds_put_cstr(s, "output:");
+            ds_put_format(s, "%soutput:%s", colors.special, colors.end);
             mf_format_subfield(&spec->src, s);
             break;
         }
     }
-    ds_put_char(s, ')');
+    ds_put_format(s, "%s)%s", colors.learn, colors.end);
 }
