@@ -39,7 +39,7 @@
 #include <sys/utsname.h>
 #endif
 #include "compiler.h"
-#include "list.h"
+#include "openvswitch/list.h"
 #include "packets.h"
 #include "timeval.h"
 
@@ -85,10 +85,10 @@ lldpd_alloc_hardware(struct lldpd *cfg, char *name, int index)
     hw->h_cfg = cfg;
     ovs_strlcpy(hw->h_ifname, name, sizeof hw->h_ifname);
     hw->h_ifindex = index;
-    hw->h_lport.p_chassis = CONTAINER_OF(list_front(&cfg->g_chassis),
+    hw->h_lport.p_chassis = CONTAINER_OF(ovs_list_front(&cfg->g_chassis),
                                          struct lldpd_chassis, list);
     hw->h_lport.p_chassis->c_refcount++;
-    list_init(&hw->h_rports);
+    ovs_list_init(&hw->h_rports);
 
     return hw;
 }
@@ -139,7 +139,7 @@ lldpd_cleanup(struct lldpd *cfg)
 
     LIST_FOR_EACH_SAFE (hw, hw_next, h_entries, &cfg->g_hardware) {
         if (!hw->h_flags) {
-            list_remove(&hw->h_entries);
+            ovs_list_remove(&hw->h_entries);
             lldpd_remote_cleanup(hw, NULL, true);
             lldpd_hardware_cleanup(cfg, hw);
         } else {
@@ -151,7 +151,7 @@ lldpd_cleanup(struct lldpd *cfg)
 
     LIST_FOR_EACH_SAFE (chassis, chassis_next, list, &cfg->g_chassis) {
         if (chassis->c_refcount == 0) {
-            list_remove(&chassis->list);
+            ovs_list_remove(&chassis->list);
             lldpd_chassis_cleanup(chassis, 1);
         }
     }
@@ -179,11 +179,11 @@ lldpd_move_chassis(struct lldpd_chassis *ochassis,
      * marshaling.
      */
     memcpy(ochassis, chassis, sizeof *ochassis);
-    list_init(&ochassis->c_mgmt);
+    ovs_list_init(&ochassis->c_mgmt);
 
     /* Copy of management addresses */
     LIST_FOR_EACH_POP (mgmt, m_entries, &chassis->c_mgmt) {
-        list_insert(&ochassis->c_mgmt, &mgmt->m_entries);
+        ovs_list_insert(&ochassis->c_mgmt, &mgmt->m_entries);
     }
 
     /* Restore saved values */
@@ -242,6 +242,7 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 
     if (s < sizeof(struct eth_header) + 4) {
         /* Too short, just discard it */
+        hw->h_rx_discarded_cnt++;
         return;
     }
 
@@ -282,6 +283,7 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
                 VLOG_DBG("function for %s protocol did not "
                          "decode this frame",
                          cfg->g_protocols[i].name);
+                hw->h_rx_discarded_cnt++;
                 return;
             }
             chassis->c_protocol = port->p_protocol = cfg->g_protocols[i].mode;
@@ -360,7 +362,7 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 
     if (oport) {
         /* The port is known, remove it before adding it back */
-        list_remove(&oport->p_entries);
+        ovs_list_remove(&oport->p_entries);
         lldpd_port_cleanup(oport, 1);
         free(oport);
     }
@@ -373,8 +375,8 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
         VLOG_DBG("unknown chassis, add it to the list");
         chassis->c_index = ++cfg->g_lastrid;
         chassis->c_refcount = 0;
-        list_push_back(&cfg->g_chassis, &chassis->list);
-        listsize = list_size(&cfg->g_chassis);
+        ovs_list_push_back(&cfg->g_chassis, &chassis->list);
+        listsize = ovs_list_size(&cfg->g_chassis);
         VLOG_DBG("%"PRIuSIZE " different systems are known", listsize);
     }
 
@@ -383,7 +385,7 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
     port->p_lastframe = xmalloc(s + sizeof(struct lldpd_frame));
     port->p_lastframe->size = s;
     memcpy(port->p_lastframe->frame, frame, s);
-    list_insert(&hw->h_rports, &port->p_entries);
+    ovs_list_insert(&hw->h_rports, &port->p_entries);
 
     port->p_chassis = chassis;
     port->p_chassis->c_refcount++;
@@ -400,7 +402,7 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
      * freed with lldpd_port_cleanup() and therefore, the refcount
      * of the chassis that was attached to it is decreased.
      */
-    i = list_size(&hw->h_rports);
+    i = ovs_list_size(&hw->h_rports);
     VLOG_DBG("%"PRIuSIZE " neighbors for %s", i, hw->h_ifname);
 
     if (!oport)  {
