@@ -23,18 +23,20 @@
 #include <stdint.h>
 #include "openvswitch/types.h"
 
-struct flow;
 struct nlattr;
-struct ofpbuf;
+struct dp_packet;
+struct pkt_metadata;
+struct dp_packet_batch;
 
-typedef void (*odp_output_cb)(void *dp, struct ofpbuf *packet,
-                              const struct flow *key, odp_port_t out_port);
-typedef void (*odp_userspace_cb)(void *dp, struct ofpbuf *packet,
-                                 const struct flow *key,
-                                 const struct nlattr *action, bool may_steal);
+typedef void (*odp_execute_cb)(void *dp, struct dp_packet_batch *batch,
+                               const struct nlattr *action, bool may_steal);
 
-void
-odp_execute_actions(void *dp, struct ofpbuf *packet, struct flow *key,
-                    const struct nlattr *actions, size_t actions_len,
-                    odp_output_cb output, odp_userspace_cb userspace);
+/* Actions that need to be executed in the context of a datapath are handed
+ * to 'dp_execute_action', if non-NULL.  Currently this is called only for
+ * actions OVS_ACTION_ATTR_OUTPUT and OVS_ACTION_ATTR_USERSPACE so
+ * 'dp_execute_action' needs to handle only these. */
+void odp_execute_actions(void *dp, struct dp_packet_batch *batch,
+                         bool steal,
+                         const struct nlattr *actions, size_t actions_len,
+                         odp_execute_cb dp_execute_action);
 #endif
