@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Nicira, Inc.
+ * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2019 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -140,7 +140,8 @@ struct rconn {
     struct vconn *monitors[MAXIMUM_MONITORS];
     size_t n_monitors;
 
-    uint32_t allowed_versions;
+    uint32_t allowed_versions;  /* Acceptable OpenFlow versions. */
+    int version;                /* Current or most recent version. */
 };
 
 uint32_t rconn_get_allowed_versions(const struct rconn *rconn)
@@ -173,8 +174,6 @@ static bool is_connected_state(enum state);
 static bool is_admitted_msg(const struct ofpbuf *);
 static bool rconn_logging_connection_attempts__(const struct rconn *rc)
     OVS_REQUIRES(rc->mutex);
-static int rconn_get_version__(const struct rconn *rconn)
-    OVS_REQUIRES(rconn->mutex);
 
 /* The following prototypes duplicate those in rconn.h, but there we weren't
  * able to add the OVS_EXCLUDED annotations because the definition of struct
@@ -275,7 +274,9 @@ rconn_create(int probe_interval, int max_backoff, uint8_t dscp,
     rconn_set_dscp(rc, dscp);
 
     rc->n_monitors = 0;
+
     rc->allowed_versions = allowed_versions;
+    rc->version = -1;
 
     return rc;
 }
@@ -367,8 +368,7 @@ rconn_connect_unreliably(struct rconn *rc,
     rconn_set_target__(rc, vconn_get_name(vconn), name);
     rc->reliable = false;
     rc->vconn = vconn;
-    rc->last_connected = time_now();
-    state_transition(rc, S_ACTIVE);
+    state_transition(rc, S_CONNECTING);
     ovs_mutex_unlock(&rc->mutex);
 }
 
@@ -493,7 +493,7 @@ static unsigned int
 timeout_CONNECTING(const struct rconn *rc)
     OVS_REQUIRES(rc->mutex)
 {
-    return MAX(1, rc->backoff);
+    return MAX(2, rc->backoff);
 }
 
 static void
@@ -502,9 +502,10 @@ run_CONNECTING(struct rconn *rc)
 {
     int retval = vconn_connect(rc->vconn);
     if (!retval) {
-        VLOG_INFO("%s: connected", rc->name);
+        VLOG(rc->reliable ? VLL_INFO : VLL_DBG, "%s: connected", rc->name);
         rc->n_successful_connections++;
         state_transition(rc, S_ACTIVE);
+        rc->version = vconn_get_version(rc->vconn);
         rc->last_connected = rc->state_entered;
     } else if (retval != EAGAIN) {
         if (rconn_logging_connection_attempts__(rc)) {
@@ -558,19 +559,17 @@ run_ACTIVE(struct rconn *rc)
 {
     if (timed_out(rc)) {
         unsigned int base = MAX(rc->last_activity, rc->state_entered);
-        int version;
-
         VLOG_DBG("%s: idle %u seconds, sending inactivity probe",
                  rc->name, (unsigned int) (time_now() - base));
 
-        version = rconn_get_version__(rc);
-        ovs_assert(version >= 0 && version <= 0xff);
-
         /* Ordering is important here: rconn_send() can transition to BACKOFF,
          * and we don't want to transition back to IDLE if so, because then we
          * can end up queuing a packet with vconn == NULL and then *boom*. */
         state_transition(rc, S_IDLE);
-        rconn_send__(rc, make_echo_request(version), NULL);
+
+        /* Send an echo request. */
+        rconn_send__(rc, make_echo_request(rc->version), NULL);
+
         return;
     }
 
@@ -931,23 +930,19 @@ rconn_failure_duration(const struct rconn *rconn)
     return duration;
 }
 
-static int
-rconn_get_version__(const struct rconn *rconn)
-    OVS_REQUIRES(rconn->mutex)
-{
-    return rconn->vconn ? vconn_get_version(rconn->vconn) : -1;
-}
-
-/* Returns the OpenFlow version negotiated with the peer, or -1 if there is
- * currently no connection or if version negotiation is not yet complete. */
+/* Returns the OpenFlow version most recently negotiated with a peer, or -1 if
+ * no version has ever been negotiated.
+ *
+ * If 'rconn' is connected (that is, if 'rconn_is_connected(rconn)' would
+ * return true), then the return value is guaranteed to be the OpenFlow version
+ * in use for the connection.  The converse is not true: when the return value
+ * is not -1, 'rconn' might be disconnected. */
 int
 rconn_get_version(const struct rconn *rconn)
     OVS_EXCLUDED(rconn->mutex)
 {
-    int version;
-
     ovs_mutex_lock(&rconn->mutex);
-    version = rconn_get_version__(rconn);
+    int version = rconn->version;
     ovs_mutex_unlock(&rconn->mutex);
 
     return version;
