@@ -25,10 +25,11 @@
 #include <net/if.h>
 
 #include "hash.h"
+#include "netdev.h"
 #include "netlink.h"
 #include "netlink-notifier.h"
 #include "netlink-socket.h"
-#include "ofpbuf.h"
+#include "openvswitch/ofpbuf.h"
 #include "ovs-router.h"
 #include "packets.h"
 #include "rtnetlink.h"
@@ -62,7 +63,6 @@ static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 20);
 static uint64_t rt_change_seq;
 
 static struct nln *nln = NULL;
-static struct nln *nln6 = NULL;
 static struct route_table_msg rtmsg;
 static struct nln_notifier *route_notifier = NULL;
 static struct nln_notifier *route6_notifier = NULL;
@@ -72,7 +72,7 @@ static bool route_table_valid = false;
 
 static int route_table_reset(void);
 static void route_table_handle_msg(const struct route_table_msg *);
-static bool route_table_parse(struct ofpbuf *, struct route_table_msg *);
+static int route_table_parse(struct ofpbuf *, struct route_table_msg *);
 static void route_table_change(const struct route_table_msg *, void *);
 static void route_map_clear(void);
 
@@ -93,22 +93,19 @@ route_table_init(void)
 {
     ovs_mutex_lock(&route_table_mutex);
     ovs_assert(!nln);
-    ovs_assert(!nln6);
     ovs_assert(!route_notifier);
     ovs_assert(!route6_notifier);
 
     ovs_router_init();
-    nln = nln_create(NETLINK_ROUTE, RTNLGRP_IPV4_ROUTE,
-                     (nln_parse_func *) route_table_parse, &rtmsg);
-    nln6 = nln_create(NETLINK_ROUTE, RTNLGRP_IPV6_ROUTE,
-                      (nln_parse_func *) route_table_parse, &rtmsg);
+    nln = nln_create(NETLINK_ROUTE, (nln_parse_func *) route_table_parse,
+                     &rtmsg);
 
     route_notifier =
-        nln_notifier_create(nln, (nln_notify_func *) route_table_change,
-                            NULL);
+        nln_notifier_create(nln, RTNLGRP_IPV4_ROUTE,
+                            (nln_notify_func *) route_table_change, NULL);
     route6_notifier =
-        nln_notifier_create(nln6, (nln_notify_func *) route_table_change,
-                            NULL);
+        nln_notifier_create(nln, RTNLGRP_IPV6_ROUTE,
+                            (nln_notify_func *) route_table_change, NULL);
 
     route_table_reset();
     name_table_init();
@@ -122,14 +119,9 @@ route_table_run(void)
     OVS_EXCLUDED(route_table_mutex)
 {
     ovs_mutex_lock(&route_table_mutex);
-    if (nln || nln6) {
+    if (nln) {
         rtnetlink_run();
-        if (nln) {
-            nln_run(nln);
-        }
-        if (nln6) {
-            nln_run(nln6);
-        }
+        nln_run(nln);
 
         if (!route_table_valid) {
             route_table_reset();
@@ -144,14 +136,9 @@ route_table_wait(void)
     OVS_EXCLUDED(route_table_mutex)
 {
     ovs_mutex_lock(&route_table_mutex);
-    if (nln || nln6) {
+    if (nln) {
         rtnetlink_wait();
-        if (nln) {
-            nln_wait(nln);
-        }
-        if (nln6) {
-            nln_wait(nln6);
-        }
+        nln_wait(nln);
     }
     ovs_mutex_unlock(&route_table_mutex);
 }
@@ -165,6 +152,7 @@ route_table_reset(void)
     struct ofpbuf request, reply, buf;
 
     route_map_clear();
+    netdev_get_addrs_list_flush();
     route_table_valid = true;
     rt_change_seq++;
 
@@ -191,14 +179,16 @@ route_table_reset(void)
     return nl_dump_done(&dump);
 }
 
-static bool
+/* Return RTNLGRP_IPV4_ROUTE or RTNLGRP_IPV6_ROUTE on success, 0 on parse
+ * error. */
+static int
 route_table_parse(struct ofpbuf *buf, struct route_table_msg *change)
 {
     bool parsed, ipv4 = false;
 
     static const struct nl_policy policy[] = {
         [RTA_DST] = { .type = NL_A_U32, .optional = true  },
-        [RTA_OIF] = { .type = NL_A_U32, .optional = false },
+        [RTA_OIF] = { .type = NL_A_U32, .optional = true },
         [RTA_GATEWAY] = { .type = NL_A_U32, .optional = true },
     };
 
@@ -222,7 +212,7 @@ route_table_parse(struct ofpbuf *buf, struct route_table_msg *change)
                                  policy6, attrs, ARRAY_SIZE(policy6));
     } else {
         VLOG_DBG_RL(&rl, "received non AF_INET rtnetlink route message");
-        return false;
+        return 0;
     }
 
     if (parsed) {
@@ -255,7 +245,7 @@ route_table_parse(struct ofpbuf *buf, struct route_table_msg *change)
                 if (error == ENXIO) {
                     change->relevant = false;
                 } else {
-                    return false;
+                    return 0;
                 }
             }
         }
@@ -280,13 +270,13 @@ route_table_parse(struct ofpbuf *buf, struct route_table_msg *change)
                 change->rd.rta_gw = nl_attr_get_in6_addr(attrs[RTA_GATEWAY]);
             }
         }
-
-
     } else {
         VLOG_DBG_RL(&rl, "received unparseable rtnetlink route message");
+        return 0;
     }
 
-    return parsed;
+    /* Success. */
+    return ipv4 ? RTNLGRP_IPV4_ROUTE : RTNLGRP_IPV6_ROUTE;
 }
 
 static void
@@ -314,11 +304,11 @@ route_map_clear(void)
 }
 
 bool
-route_table_fallback_lookup(ovs_be32 ip_dst OVS_UNUSED,
-                            char output_bridge[] OVS_UNUSED,
-                            ovs_be32 *gw)
+route_table_fallback_lookup(const struct in6_addr *ip6_dst OVS_UNUSED,
+                            char name[] OVS_UNUSED,
+                            struct in6_addr *gw6)
 {
-    *gw = 0;
+    *gw6 = in6addr_any;
     return false;
 }
 
