@@ -27,7 +27,7 @@
 #include <stdlib.h>
 #include "byte-order.h"
 #include "connectivity.h"
-#include "ofpbuf.h"
+#include "openvswitch/ofpbuf.h"
 #include "ovs-atomic.h"
 #include "dp-packet.h"
 #include "packets.h"
@@ -302,7 +302,7 @@ stp_create(const char *name, stp_identifier bridge_id,
 
     stp_stop_timer(&stp->tcn_timer);
     stp_stop_timer(&stp->topology_change_timer);
-    stp_start_timer(&stp->hello_timer, 0);
+    stp_start_timer(&stp->hello_timer, stp->hello_time);
 
     stp->send_bpdu = send_bpdu;
     stp->aux = aux;
@@ -316,7 +316,7 @@ stp_create(const char *name, stp_identifier bridge_id,
     }
     ovs_refcount_init(&stp->ref_cnt);
 
-    list_push_back(all_stps, &stp->node);
+    ovs_list_push_back(all_stps, &stp->node);
     ovs_mutex_unlock(&mutex);
     return stp;
 }
@@ -339,7 +339,7 @@ stp_unref(struct stp *stp)
         size_t i;
 
         ovs_mutex_lock(&mutex);
-        list_remove(&stp->node);
+        ovs_list_remove(&stp->node);
         ovs_mutex_unlock(&mutex);
         free(stp->name);
 
