@@ -880,7 +880,8 @@ sflow_read_tnl_push_action(const struct nlattr *attr,
 {
     /* Modeled on lib/odp-util.c: format_odp_tnl_push_header */
     const struct ovs_action_push_tnl *data = nl_attr_get(attr);
-    const struct eth_header *eth = (const struct eth_header *) data->header;
+    const struct eth_header *eth = ALIGNED_CAST(const struct eth_header *,
+                                                data->header);
     const struct ip_header *ip
         = ALIGNED_CAST(const struct ip_header *, eth + 1);
 
@@ -996,7 +997,7 @@ sflow_read_set_action(const struct nlattr *attr,
                 sflow_actions->tunnel.ip_tos = key->ipv4_tos;
             }
             if (key->ipv4_ttl) {
-                sflow_actions->tunnel.ip_tos = key->ipv4_ttl;
+                sflow_actions->tunnel.ip_ttl = key->ipv4_ttl;
             }
         }
         break;
