@@ -581,6 +581,10 @@ make_cmp(struct expr_context *ctx,
         }
     }
 
+    if (!cs->n_values) {
+        e = expr_create_boolean(r == EXPR_R_NE);
+        goto exit;
+    }
     e = make_cmp__(f, r, &cs->values[0]);
     for (size_t i = 1; i < cs->n_values; i++) {
         e = expr_combine(r == EXPR_R_EQ ? EXPR_T_OR : EXPR_T_AND,
@@ -1668,6 +1672,18 @@ expr_annotate(struct expr *expr, const struct shash *symtab, char **errorp)
     return expr_annotate__(expr, symtab, &nesting, errorp);
 }
 
+static struct expr *
+expr_simplify_eq(struct expr *expr)
+{
+    const union mf_subvalue *mask = &expr->cmp.mask;
+    if (is_all_zeros(mask, sizeof *mask)) {
+        /* Simplify "ip4.dst == 0/0" to just "1" (plus a prerequisite). */
+        expr_destroy(expr);
+        return expr_create_boolean(true);
+    }
+    return expr;
+}
+
 static struct expr *
 expr_simplify_ne(struct expr *expr)
 {
@@ -1690,7 +1706,16 @@ expr_simplify_ne(struct expr *expr)
 
         new = expr_combine(EXPR_T_OR, new, e);
     }
-    ovs_assert(new);
+    if (!new) {
+        /* Handle a comparison like "ip4.dst != 0/0", where the mask has no
+         * 1-bits.
+         *
+         * The correct result for this expression may not be obvious.  It's
+         * easier to understand that "ip4.dst == 0/0" should be true, since 0/0
+         * matches every IPv4 address; then, "ip4.dst != 0/0" should have the
+         * opposite result. */
+        new = expr_create_boolean(false);
+    }
 
     expr_destroy(expr);
 
@@ -1742,7 +1767,8 @@ expr_simplify(struct expr *expr)
 
     switch (expr->type) {
     case EXPR_T_CMP:
-        return (expr->cmp.relop == EXPR_R_EQ || !expr->cmp.symbol->width ? expr
+        return (!expr->cmp.symbol->width ? expr
+                : expr->cmp.relop == EXPR_R_EQ ? expr_simplify_eq(expr)
                 : expr->cmp.relop == EXPR_R_NE ? expr_simplify_ne(expr)
                 : expr_simplify_relational(expr));
 
