@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011, 2012, 2013, 2014 Nicira, Inc.
+/* Copyright (c) 2009, 2010, 2011, 2012, 2013, 2014, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,24 +26,26 @@
 #include "command-line.h"
 #include "daemon.h"
 #include "dirs.h"
-#include "dynamic-string.h"
+#include "openvswitch/dynamic-string.h"
 #include "fatal-signal.h"
 #include "file.h"
 #include "hash.h"
-#include "json.h"
+#include "openvswitch/json.h"
 #include "jsonrpc.h"
 #include "jsonrpc-server.h"
-#include "list.h"
+#include "openvswitch/list.h"
 #include "memory.h"
+#include "monitor.h"
 #include "ovsdb.h"
 #include "ovsdb-data.h"
 #include "ovsdb-types.h"
 #include "ovsdb-error.h"
 #include "poll-loop.h"
 #include "process.h"
+#include "replication.h"
 #include "row.h"
 #include "simap.h"
-#include "shash.h"
+#include "openvswitch/shash.h"
 #include "stream-ssl.h"
 #include "stream.h"
 #include "sset.h"
@@ -79,11 +81,22 @@ static unixctl_cb_func ovsdb_server_compact;
 static unixctl_cb_func ovsdb_server_reconnect;
 static unixctl_cb_func ovsdb_server_perf_counters_clear;
 static unixctl_cb_func ovsdb_server_perf_counters_show;
+static unixctl_cb_func ovsdb_server_disable_monitor_cond;
+static unixctl_cb_func ovsdb_server_set_active_ovsdb_server;
+static unixctl_cb_func ovsdb_server_get_active_ovsdb_server;
+static unixctl_cb_func ovsdb_server_connect_active_ovsdb_server;
+static unixctl_cb_func ovsdb_server_disconnect_active_ovsdb_server;
+static unixctl_cb_func ovsdb_server_set_sync_exclude_tables;
+static unixctl_cb_func ovsdb_server_get_sync_exclude_tables;
+static unixctl_cb_func ovsdb_server_get_sync_status;
 
 struct server_config {
     struct sset *remotes;
     struct shash *all_dbs;
     FILE *config_tmpfile;
+    char **sync_from;
+    char **sync_exclude;
+    bool *is_backup;
     struct ovsdb_jsonrpc_server *jsonrpc;
 };
 static unixctl_cb_func ovsdb_server_add_remote;
@@ -99,7 +112,8 @@ static void close_db(struct db *db);
 
 static void parse_options(int *argc, char **argvp[],
                           struct sset *remotes, char **unixctl_pathp,
-                          char **run_command);
+                          char **run_command, char **sync_from,
+                          char **sync_exclude, bool *is_backup);
 OVS_NO_RETURN static void usage(void);
 
 static char *reconfigure_remotes(struct ovsdb_jsonrpc_server *,
@@ -113,19 +127,35 @@ static void update_remote_status(const struct ovsdb_jsonrpc_server *jsonrpc,
                                  struct shash *all_dbs);
 
 static void save_config__(FILE *config_file, const struct sset *remotes,
-                          const struct sset *db_filenames);
+                          const struct sset *db_filenames,
+                          const char *sync_from, const char *sync_exclude,
+                          bool is_backup);
 static void save_config(struct server_config *);
 static void load_config(FILE *config_file, struct sset *remotes,
-                        struct sset *db_filenames);
+                        struct sset *db_filenames, char **sync_from,
+                        char **sync_exclude, bool *is_backup);
+
+static void
+ovsdb_replication_init(const char *sync_from, const char *exclude,
+                       struct shash *all_dbs)
+{
+    replication_init(sync_from, exclude);
+    struct shash_node *node;
+    SHASH_FOR_EACH (node, all_dbs) {
+        struct db *db = node->data;
+        replication_add_local_db(db->db->schema->name, db->db);
+    }
+}
 
 static void
 main_loop(struct ovsdb_jsonrpc_server *jsonrpc, struct shash *all_dbs,
           struct unixctl_server *unixctl, struct sset *remotes,
-          struct process *run_process, bool *exiting)
+          struct process *run_process, bool *exiting, bool *is_backup)
 {
     char *remotes_error, *ssl_error;
     struct shash_node *node;
     long long int status_timer = LLONG_MIN;
+    bool last_role = *is_backup;
 
     *exiting = false;
     ssl_error = NULL;
@@ -137,6 +167,7 @@ main_loop(struct ovsdb_jsonrpc_server *jsonrpc, struct shash *all_dbs,
 
             simap_init(&usage);
             ovsdb_jsonrpc_server_get_memory_usage(jsonrpc, &usage);
+            ovsdb_monitor_get_memory_usage(&usage);
             SHASH_FOR_EACH(node, all_dbs) {
                 struct db *db = node->data;
                 ovsdb_get_memory_usage(db->db, &usage);
@@ -150,12 +181,27 @@ main_loop(struct ovsdb_jsonrpc_server *jsonrpc, struct shash *all_dbs,
          * the set of remotes that reconfigure_remotes() uses. */
         unixctl_server_run(unixctl);
 
+        /* In ovsdb-server's role (active or backup) has changed, restart
+         * the ovsdb jsonrpc server.  */
+        if (last_role != *is_backup) {
+            bool read_only = last_role = *is_backup;
+            ovsdb_jsonrpc_server_reconnect(jsonrpc, read_only);
+        }
+
         report_error_if_changed(
             reconfigure_remotes(jsonrpc, all_dbs, remotes),
             &remotes_error);
         report_error_if_changed(reconfigure_ssl(all_dbs), &ssl_error);
         ovsdb_jsonrpc_server_run(jsonrpc);
 
+        if (*is_backup) {
+            replication_run();
+            if (!replication_is_alive()) {
+                int retval = replication_get_last_error();
+                ovs_fatal(retval, "replication connection failed");
+            }
+        }
+
         SHASH_FOR_EACH(node, all_dbs) {
             struct db *db = node->data;
             ovsdb_trigger_run(db->db, time_msec());
@@ -167,13 +213,17 @@ main_loop(struct ovsdb_jsonrpc_server *jsonrpc, struct shash *all_dbs,
             }
         }
 
-        /* update Manager status(es) every 5 seconds */
+        /* update Manager status(es) every 2.5 seconds */
         if (time_msec() >= status_timer) {
-            status_timer = time_msec() + 5000;
+            status_timer = time_msec() + 2500;
             update_remote_status(jsonrpc, remotes, all_dbs);
         }
 
         memory_wait();
+        if (*is_backup) {
+            replication_wait();
+        }
+
         ovsdb_jsonrpc_server_wait(jsonrpc);
         unixctl_server_wait(unixctl);
         SHASH_FOR_EACH(node, all_dbs) {
@@ -193,6 +243,7 @@ main_loop(struct ovsdb_jsonrpc_server *jsonrpc, struct shash *all_dbs,
         }
     }
 
+    free(remotes_error);
 }
 
 int
@@ -203,6 +254,8 @@ main(int argc, char *argv[])
     struct unixctl_server *unixctl;
     struct ovsdb_jsonrpc_server *jsonrpc;
     struct sset remotes, db_filenames;
+    char *sync_from, *sync_exclude;
+    bool is_backup;
     const char *db_filename;
     struct process *run_process;
     bool exiting;
@@ -220,7 +273,11 @@ main(int argc, char *argv[])
     fatal_ignore_sigpipe();
     process_init();
 
-    parse_options(&argc, &argv, &remotes, &unixctl_path, &run_command);
+    bool active = false;
+    parse_options(&argc, &argv, &remotes, &unixctl_path, &run_command,
+                  &sync_from, &sync_exclude, &active);
+    is_backup = sync_from && !active;
+
     daemon_become_new_user(false);
 
     /* Create and initialize 'config_tmpfile' as a temporary file to hold
@@ -247,17 +304,26 @@ main(int argc, char *argv[])
     server_config.remotes = &remotes;
     server_config.config_tmpfile = config_tmpfile;
 
-    save_config__(config_tmpfile, &remotes, &db_filenames);
+    save_config__(config_tmpfile, &remotes, &db_filenames, sync_from,
+                  sync_exclude, is_backup);
 
     daemonize_start(false);
 
     /* Load the saved config. */
-    load_config(config_tmpfile, &remotes, &db_filenames);
-    jsonrpc = ovsdb_jsonrpc_server_create();
+    load_config(config_tmpfile, &remotes, &db_filenames, &sync_from,
+                &sync_exclude, &is_backup);
+
+    /* Start ovsdb jsonrpc server. When running as a backup server,
+     * jsonrpc connections are read only. Otherwise, both read
+     * and write transactions are allowed.  */
+    jsonrpc = ovsdb_jsonrpc_server_create(is_backup);
 
     shash_init(&all_dbs);
     server_config.all_dbs = &all_dbs;
     server_config.jsonrpc = jsonrpc;
+    server_config.sync_from = &sync_from;
+    server_config.sync_exclude = &sync_exclude;
+    server_config.is_backup = &is_backup;
 
     perf_counters_init();
 
@@ -329,8 +395,39 @@ main(int argc, char *argv[])
                              ovsdb_server_perf_counters_show, NULL);
     unixctl_command_register("ovsdb-server/perf-counters-clear", "", 0, 0,
                              ovsdb_server_perf_counters_clear, NULL);
+    unixctl_command_register("ovsdb-server/set-active-ovsdb-server", "", 1, 1,
+                             ovsdb_server_set_active_ovsdb_server,
+                             &server_config);
+    unixctl_command_register("ovsdb-server/get-active-ovsdb-server", "", 0, 0,
+                             ovsdb_server_get_active_ovsdb_server,
+                             &server_config);
+    unixctl_command_register("ovsdb-server/connect-active-ovsdb-server", "",
+                             0, 0, ovsdb_server_connect_active_ovsdb_server,
+                             &server_config);
+    unixctl_command_register("ovsdb-server/disconnect-active-ovsdb-server", "",
+                             0, 0, ovsdb_server_disconnect_active_ovsdb_server,
+                             &server_config);
+    unixctl_command_register("ovsdb-server/set-sync-exclude-tables", "",
+                             0, 1, ovsdb_server_set_sync_exclude_tables,
+                             &server_config);
+    unixctl_command_register("ovsdb-server/get-sync-exclude-tables", "",
+                             0, 0, ovsdb_server_get_sync_exclude_tables,
+                             NULL);
+    unixctl_command_register("ovsdb-server/sync-status", "",
+                             0, 0, ovsdb_server_get_sync_status,
+                             &server_config);
 
-    main_loop(jsonrpc, &all_dbs, unixctl, &remotes, run_process, &exiting);
+    /* Simulate the behavior of OVS release prior to version 2.5 that
+     * does not support the monitor_cond method.  */
+    unixctl_command_register("ovsdb-server/disable-monitor-cond", "", 0, 0,
+                             ovsdb_server_disable_monitor_cond, jsonrpc);
+
+    if (is_backup) {
+        ovsdb_replication_init(sync_from, sync_exclude, &all_dbs);
+    }
+
+    main_loop(jsonrpc, &all_dbs, unixctl, &remotes, run_process, &exiting,
+              &is_backup);
 
     ovsdb_jsonrpc_server_destroy(jsonrpc);
     SHASH_FOR_EACH_SAFE(node, next, &all_dbs) {
@@ -341,7 +438,10 @@ main(int argc, char *argv[])
     shash_destroy(&all_dbs);
     sset_destroy(&remotes);
     sset_destroy(&db_filenames);
+    free(sync_from);
+    free(sync_exclude);
     unixctl_server_destroy(unixctl);
+    replication_destroy();
 
     if (run_process && process_exited(run_process)) {
         int status = process_status(run_process);
@@ -430,7 +530,7 @@ find_db(const struct shash *all_dbs, const char *db_name)
 {
     struct shash_node *node;
 
-    SHASH_FOR_EACH(node, all_dbs) {
+    SHASH_FOR_EACH (node, all_dbs) {
         struct db *db = node->data;
         if (!strcmp(db->db->schema->name, db_name)) {
             return db;
@@ -1023,6 +1123,103 @@ report_error_if_changed(char *error, char **last_errorp)
     }
 }
 
+static void
+ovsdb_server_set_active_ovsdb_server(struct unixctl_conn *conn,
+                                     int argc OVS_UNUSED, const char *argv[],
+                                     void *config_)
+{
+    struct server_config *config = config_;
+
+    if (*config->sync_from) {
+        free(*config->sync_from);
+    }
+    *config->sync_from = xstrdup(argv[1]);
+    save_config(config);
+
+    unixctl_command_reply(conn, NULL);
+}
+
+static void
+ovsdb_server_get_active_ovsdb_server(struct unixctl_conn *conn,
+                                     int argc OVS_UNUSED,
+                                     const char *argv[] OVS_UNUSED,
+                                     void *config_ )
+{
+    struct server_config *config = config_;
+
+    unixctl_command_reply(conn, *config->sync_from);
+}
+
+static void
+ovsdb_server_connect_active_ovsdb_server(struct unixctl_conn *conn,
+                                         int argc OVS_UNUSED,
+                                         const char *argv[] OVS_UNUSED,
+                                         void *config_)
+{
+    struct server_config *config = config_;
+    char *msg = NULL;
+
+    if ( !*config->sync_from) {
+        msg = "Unable to connect: active server is not specified.\n";
+    } else {
+        ovsdb_replication_init(*config->sync_from, *config->sync_exclude,
+                               config->all_dbs);
+        if (!*config->is_backup) {
+            *config->is_backup = true;
+            save_config(config);
+        }
+    }
+    unixctl_command_reply(conn, msg);
+}
+
+static void
+ovsdb_server_disconnect_active_ovsdb_server(struct unixctl_conn *conn,
+                                            int argc OVS_UNUSED,
+                                            const char *argv[] OVS_UNUSED,
+                                            void *config_)
+{
+    struct server_config *config = config_;
+
+    disconnect_active_server();
+    *config->is_backup = false;
+    save_config(config);
+    unixctl_command_reply(conn, NULL);
+}
+
+static void
+ovsdb_server_set_sync_exclude_tables(struct unixctl_conn *conn,
+                                     int argc OVS_UNUSED,
+                                     const char *argv[],
+                                     void *config_)
+{
+    struct server_config *config = config_;
+
+    char *err = set_blacklist_tables(argv[1], true);
+    if (!err) {
+        free(*config->sync_exclude);
+        *config->sync_exclude = xstrdup(argv[1]);
+        save_config(config);
+        if (*config->is_backup) {
+            ovsdb_replication_init(*config->sync_from, *config->sync_exclude,
+                                   config->all_dbs);
+        }
+        err = set_blacklist_tables(argv[1], false);
+    }
+    unixctl_command_reply(conn, err);
+    free(err);
+}
+
+static void
+ovsdb_server_get_sync_exclude_tables(struct unixctl_conn *conn,
+                                     int argc OVS_UNUSED,
+                                     const char *argv[] OVS_UNUSED,
+                                     void *arg_ OVS_UNUSED)
+{
+    char *reply = get_blacklist_tables();
+    unixctl_command_reply(conn, reply);
+    free(reply);
+}
+
 static void
 ovsdb_server_exit(struct unixctl_conn *conn, int argc OVS_UNUSED,
                   const char *argv[] OVS_UNUSED,
@@ -1053,6 +1250,23 @@ ovsdb_server_perf_counters_clear(struct unixctl_conn *conn, int argc OVS_UNUSED,
     unixctl_command_reply(conn, NULL);
 }
 
+/* "ovsdb-server/disable-monitor-cond": makes ovsdb-server drop all of its
+ * JSON-RPC connections and reconnect. New sessions will not recognize
+ * the 'monitor_cond' method.   */
+static void
+ovsdb_server_disable_monitor_cond(struct unixctl_conn *conn,
+                                  int argc OVS_UNUSED,
+                                  const char *argv[] OVS_UNUSED,
+                                  void *jsonrpc_)
+{
+    struct ovsdb_jsonrpc_server *jsonrpc = jsonrpc_;
+    bool read_only = ovsdb_jsonrpc_server_is_read_only(jsonrpc);
+
+    ovsdb_jsonrpc_disable_monitor_cond();
+    ovsdb_jsonrpc_server_reconnect(jsonrpc, read_only);
+    unixctl_command_reply(conn, NULL);
+}
+
 static void
 ovsdb_server_compact(struct unixctl_conn *conn, int argc,
                      const char *argv[], void *dbs_)
@@ -1104,8 +1318,9 @@ ovsdb_server_reconnect(struct unixctl_conn *conn, int argc OVS_UNUSED,
                        const char *argv[] OVS_UNUSED, void *jsonrpc_)
 {
     struct ovsdb_jsonrpc_server *jsonrpc = jsonrpc_;
+    bool read_only = ovsdb_jsonrpc_server_is_read_only(jsonrpc);
 
-    ovsdb_jsonrpc_server_reconnect(jsonrpc);
+    ovsdb_jsonrpc_server_reconnect(jsonrpc, read_only);
     unixctl_command_reply(conn, NULL);
 }
 
@@ -1191,6 +1406,10 @@ ovsdb_server_add_database(struct unixctl_conn *conn, int argc OVS_UNUSED,
     error = open_db(config, filename);
     if (!error) {
         save_config(config);
+        if (*config->is_backup) {
+            ovsdb_replication_init(*config->sync_from, *config->sync_exclude,
+                                   config->all_dbs);
+        }
         unixctl_command_reply(conn, NULL);
     } else {
         unixctl_command_reply_error(conn, error);
@@ -1221,6 +1440,10 @@ ovsdb_server_remove_database(struct unixctl_conn *conn, int argc OVS_UNUSED,
     shash_delete(config->all_dbs, node);
 
     save_config(config);
+    if (*config->is_backup) {
+        ovsdb_replication_init(*config->sync_from, *config->sync_exclude,
+                               config->all_dbs);
+    }
     unixctl_command_reply(conn, NULL);
 }
 
@@ -1246,9 +1469,28 @@ ovsdb_server_list_databases(struct unixctl_conn *conn, int argc OVS_UNUSED,
     ds_destroy(&s);
 }
 
+static void
+ovsdb_server_get_sync_status(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                             const char *argv[] OVS_UNUSED, void *config_)
+{
+    struct server_config *config = config_;
+    bool is_backup = *config->is_backup;
+    struct ds ds = DS_EMPTY_INITIALIZER;
+
+    ds_put_format(&ds, "state: %s\n", is_backup ? "backup" : "active");
+
+    if (is_backup) {
+        ds_put_and_free_cstr(&ds, replication_status());
+    }
+
+    unixctl_command_reply(conn, ds_cstr(&ds));
+    ds_destroy(&ds);
+}
+
 static void
 parse_options(int *argcp, char **argvp[],
-              struct sset *remotes, char **unixctl_pathp, char **run_command)
+              struct sset *remotes, char **unixctl_pathp, char **run_command,
+              char **sync_from, char **sync_exclude, bool *active)
 {
     enum {
         OPT_REMOTE = UCHAR_MAX + 1,
@@ -1256,6 +1498,9 @@ parse_options(int *argcp, char **argvp[],
         OPT_RUN,
         OPT_BOOTSTRAP_CA_CERT,
         OPT_PEER_CA_CERT,
+        OPT_SYNC_FROM,
+        OPT_SYNC_EXCLUDE,
+        OPT_ACTIVE,
         VLOG_OPTION_ENUMS,
         DAEMON_OPTION_ENUMS
     };
@@ -1274,12 +1519,17 @@ parse_options(int *argcp, char **argvp[],
         {"private-key", required_argument, NULL, 'p'},
         {"certificate", required_argument, NULL, 'c'},
         {"ca-cert",     required_argument, NULL, 'C'},
+        {"sync-from",   required_argument, NULL, OPT_SYNC_FROM},
+        {"sync-exclude-tables", required_argument, NULL, OPT_SYNC_EXCLUDE},
+        {"active", no_argument, NULL, OPT_ACTIVE},
         {NULL, 0, NULL, 0},
     };
     char *short_options = ovs_cmdl_long_options_to_short_options(long_options);
     int argc = *argcp;
     char **argv = *argvp;
 
+    *sync_from = NULL;
+    *sync_exclude = NULL;
     sset_init(remotes);
     for (;;) {
         int c;
@@ -1334,6 +1584,22 @@ parse_options(int *argcp, char **argvp[],
             stream_ssl_set_peer_ca_cert_file(optarg);
             break;
 
+        case OPT_SYNC_FROM:
+            *sync_from = xstrdup(optarg);
+            break;
+
+        case OPT_SYNC_EXCLUDE: {
+            char *err = set_blacklist_tables(optarg, false);
+            if (err) {
+                ovs_fatal(0, "%s", err);
+            }
+            *sync_exclude = xstrdup(optarg);
+            break;
+        }
+        case OPT_ACTIVE:
+            *active = true;
+            break;
+
         case '?':
             exit(EXIT_FAILURE);
 
@@ -1360,6 +1626,7 @@ usage(void)
     stream_usage("JSON-RPC", true, true, true);
     daemon_usage();
     vlog_usage();
+    replication_usage();
     printf("\nOther options:\n"
            "  --run COMMAND           run COMMAND as subprocess then exit\n"
            "  --unixctl=SOCKET        override default control socket name\n"
@@ -1385,7 +1652,8 @@ sset_to_json(const struct sset *sset)
  * 'remotes' and 'db_filenames'. */
 static void
 save_config__(FILE *config_file, const struct sset *remotes,
-              const struct sset *db_filenames)
+              const struct sset *db_filenames, const char *sync_from,
+              const char *sync_exclude, bool is_backup)
 {
     struct json *obj;
     char *s;
@@ -1398,6 +1666,15 @@ save_config__(FILE *config_file, const struct sset *remotes,
     obj = json_object_create();
     json_object_put(obj, "remotes", sset_to_json(remotes));
     json_object_put(obj, "db_filenames", sset_to_json(db_filenames));
+    if (sync_from) {
+        json_object_put(obj, "sync_from", json_string_create(sync_from));
+    }
+    if (sync_exclude) {
+        json_object_put(obj, "sync_exclude",
+                        json_string_create(sync_exclude));
+    }
+    json_object_put(obj, "is_backup", json_boolean_create(is_backup));
+
     s = json_to_string(obj, 0);
     json_destroy(obj);
 
@@ -1423,7 +1700,9 @@ save_config(struct server_config *config)
         sset_add(&db_filenames, db->filename);
     }
 
-    save_config__(config->config_tmpfile, config->remotes, &db_filenames);
+    save_config__(config->config_tmpfile, config->remotes, &db_filenames,
+                  *config->sync_from, *config->sync_exclude,
+                  *config->is_backup);
 
     sset_destroy(&db_filenames);
 }
@@ -1445,7 +1724,8 @@ sset_from_json(struct sset *sset, const struct json *array)
 /* Clears and replaces 'remotes' and 'dbnames' by a configuration read from
  * 'config_file', which must have been previously written by save_config(). */
 static void
-load_config(FILE *config_file, struct sset *remotes, struct sset *db_filenames)
+load_config(FILE *config_file, struct sset *remotes, struct sset *db_filenames,
+            char **sync_from, char **sync_exclude, bool *is_backup)
 {
     struct json *json;
 
@@ -1461,5 +1741,17 @@ load_config(FILE *config_file, struct sset *remotes, struct sset *db_filenames)
     sset_from_json(remotes, shash_find_data(json_object(json), "remotes"));
     sset_from_json(db_filenames,
                    shash_find_data(json_object(json), "db_filenames"));
+
+    struct json *string;
+    string = shash_find_data(json_object(json), "sync_from");
+    free(*sync_from);
+    *sync_from = string ? xstrdup(json_string(string)) : NULL;
+
+    string = shash_find_data(json_object(json), "sync_exclude");
+    free(*sync_exclude);
+    *sync_exclude = string ? xstrdup(json_string(string)) : NULL;
+
+    *is_backup = json_boolean(shash_find_data(json_object(json), "is_backup"));
+
     json_destroy(json);
 }
