@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011, 2012 Nicira, Inc.
+/* Copyright (c) 2009, 2010, 2011, 2012, 2013 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@
 #include "ovsdb.h"
 
 #include "column.h"
-#include "json.h"
+#include "openvswitch/json.h"
 #include "ovsdb-error.h"
 #include "ovsdb-parser.h"
 #include "ovsdb-types.h"
@@ -103,7 +103,7 @@ ovsdb_schema_from_file(const char *file_name, struct ovsdb_schema **schemap)
     return NULL;
 }
 
-static struct ovsdb_error * WARN_UNUSED_RESULT
+static struct ovsdb_error * OVS_WARN_UNUSED_RESULT
 ovsdb_schema_check_ref_table(struct ovsdb_column *column,
                              const struct shash *tables,
                              const struct ovsdb_base_type *base,
@@ -140,7 +140,7 @@ static bool
 is_valid_version(const char *s)
 {
     int n = -1;
-    ignore(sscanf(s, "%*[0-9].%*[0-9].%*[0-9]%n", &n));
+    ignore(ovs_scan(s, "%*[0-9].%*[0-9].%*[0-9]%n", &n));
     return n != -1 && s[n] == '\0';
 }
 
@@ -328,8 +328,8 @@ ovsdb_create(struct ovsdb_schema *schema)
 
     db = xmalloc(sizeof *db);
     db->schema = schema;
-    list_init(&db->replicas);
-    list_init(&db->triggers);
+    ovs_list_init(&db->replicas);
+    ovs_list_init(&db->triggers);
     db->run_triggers = false;
 
     shash_init(&db->tables);
@@ -361,9 +361,9 @@ ovsdb_destroy(struct ovsdb *db)
         struct shash_node *node;
 
         /* Remove all the replicas. */
-        while (!list_is_empty(&db->replicas)) {
+        while (!ovs_list_is_empty(&db->replicas)) {
             struct ovsdb_replica *r
-                = CONTAINER_OF(list_pop_back(&db->replicas),
+                = CONTAINER_OF(ovs_list_pop_back(&db->replicas),
                                struct ovsdb_replica, node);
             ovsdb_remove_replica(db, r);
         }
@@ -420,12 +420,12 @@ ovsdb_replica_init(struct ovsdb_replica *r,
 void
 ovsdb_add_replica(struct ovsdb *db, struct ovsdb_replica *r)
 {
-    list_push_back(&db->replicas, &r->node);
+    ovs_list_push_back(&db->replicas, &r->node);
 }
 
 void
 ovsdb_remove_replica(struct ovsdb *db OVS_UNUSED, struct ovsdb_replica *r)
 {
-    list_remove(&r->node);
+    ovs_list_remove(&r->node);
     (r->class->destroy)(r);
 }
