@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011 Nicira Networks
+/* Copyright (c) 2009, 2010, 2011, 2012 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,10 +17,9 @@
 
 #include "trigger.h"
 
-#include <assert.h>
 #include <limits.h>
 
-#include "json.h"
+#include "openvswitch/json.h"
 #include "jsonrpc.h"
 #include "ovsdb.h"
 #include "poll-loop.h"
@@ -30,23 +29,26 @@ static bool ovsdb_trigger_try(struct ovsdb_trigger *, long long int now);
 static void ovsdb_trigger_complete(struct ovsdb_trigger *);
 
 void
-ovsdb_trigger_init(struct ovsdb_session *session,
+ovsdb_trigger_init(struct ovsdb_session *session, struct ovsdb *db,
                    struct ovsdb_trigger *trigger,
-                   struct json *request, long long int now)
+                   struct json *request, long long int now,
+                   bool read_only)
 {
     trigger->session = session;
-    list_push_back(&trigger->session->db->triggers, &trigger->node);
+    trigger->db = db;
+    ovs_list_push_back(&trigger->db->triggers, &trigger->node);
     trigger->request = request;
     trigger->result = NULL;
     trigger->created = now;
     trigger->timeout_msec = LLONG_MAX;
+    trigger->read_only = read_only;
     ovsdb_trigger_try(trigger, now);
 }
 
 void
 ovsdb_trigger_destroy(struct ovsdb_trigger *trigger)
 {
-    list_remove(&trigger->node);
+    ovs_list_remove(&trigger->node);
     json_destroy(trigger->request);
     json_destroy(trigger->result);
 }
@@ -110,8 +112,9 @@ ovsdb_trigger_wait(struct ovsdb *db, long long int now)
 static bool
 ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
 {
-    t->result = ovsdb_execute(t->session->db, t->session,
-                              t->request, now - t->created, &t->timeout_msec);
+    t->result = ovsdb_execute(t->db, t->session,
+                              t->request, t->read_only,
+                              now - t->created, &t->timeout_msec);
     if (t->result) {
         ovsdb_trigger_complete(t);
         return true;
@@ -123,7 +126,7 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
 static void
 ovsdb_trigger_complete(struct ovsdb_trigger *t)
 {
-    assert(t->result != NULL);
-    list_remove(&t->node);
-    list_push_back(&t->session->completions, &t->node);
+    ovs_assert(t->result != NULL);
+    ovs_list_remove(&t->node);
+    ovs_list_push_back(&t->session->completions, &t->node);
 }
