@@ -201,10 +201,12 @@ create_macros(struct shash *macros)
     static const char *const addrs3[] = {
         "00:00:00:00:00:01", "00:00:00:00:00:02", "00:00:00:00:00:03",
     };
+    static const char *const addrs4[] = { NULL };
 
     expr_macros_add(macros, "set1", addrs1, 3);
     expr_macros_add(macros, "set2", addrs2, 3);
     expr_macros_add(macros, "set3", addrs3, 3);
+    expr_macros_add(macros, "set4", addrs4, 0);
 }
 
 static bool
@@ -452,11 +454,11 @@ next_composition(unsigned int *state, int s[], int sn)
             j++;
         } else {
             j--;
-            s[j] = s[j + 1];
-            s[j - 1]++;
             if (!j) {
                 return 0;
             }
+            s[j] = s[j + 1];
+            s[j - 1]++;
         }
     }
     return j + 1;
@@ -626,7 +628,7 @@ init_terminal(struct expr *expr, int phase,
         memset(&expr->cmp.value, 0, sizeof expr->cmp.value);
         memset(&expr->cmp.mask, 0, sizeof expr->cmp.mask);
         expr->cmp.value.integer = htonll(0);
-        expr->cmp.mask.integer = htonll(1);
+        expr->cmp.mask.integer = htonll(0);
         return;
     }
 
@@ -721,15 +723,13 @@ next_terminal(struct expr *expr,
                     return true;
                 }
             }
-            next = 0;
-        } else if (m == 0) {
-            /* Skip: empty mask is pathological. */
+            next = UINT_MAX;
         } else if (v & ~m) {
             /* Skip: 1-bits in value correspond to 0-bits in mask. */
-        } else if (turn_off_rightmost_1s(m)
+        } else if ((!m || turn_off_rightmost_1s(m))
                    && (expr->cmp.relop != EXPR_R_EQ &&
                        expr->cmp.relop != EXPR_R_NE)) {
-            /* Skip: can't have discontiguous mask for > >= < <=. */
+            /* Skip: can't have discontiguous or all-0 mask for > >= < <=. */
         } else {
             expr->cmp.value.integer = htonll(v);
             expr->cmp.mask.integer = htonll(m);
